<?php
/*
Plugin Name: Google Latitude History
Plugin URI: http://worldtravelblog.com/code/google-latitude-history-plugin/
Description: This plugin allows you to display your Google Latitude location history on Google Maps
Version: 0.9.1
Author: Peter Rosanelli
Author URI: http://www.worldtravelblog.com
*/

/**
* LICENSE
* This file is part of Google Latitude History Plugin.
*
* Google Latitude History Plugin is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*
* @package    google-latitude-history
* @author     Peter Rosanelli <peter@worldtravelblog.com>
* @copyright  Copyright 2011 Peter Rosanelli
* @license    http://www.gnu.org/licenses/gpl.txt GPL 2.0
* @version    0.9.1
* @link       http://worldtravelblog.com/code/google-latitude-history-plugin/
*/


@session_start();
if(!class_exists('LatitudeOAuth')) {
	require_once('latitudeoauth/latitudeoauth.php');
}

global $wpdb;

// settings page
require_once('settings.php');
// generates map from shortcode
require_once('google-latitude-history-map.php');

if(class_exists('GoogleLatitudeHistory')) {
	$plugin_glatitudehistory = new GoogleLatitudeHistory();
}

if(isset($plugin_glatitudehistory)) {

	/**
	 * Creates plugin option page
	 * This is outside of the class b/c syncLocations()
	 * which is called from options page will fail. it fails
	 * b/c $this is called outside of the object context. 
	 */
	function glatitudehistory_admin_menu() {
		global $plugin_glatitudehistory;
		add_options_page('Google Latitude History Settings', 'Google Latitude History', 'manage_options', 'google-latitude-history-menu', array(&$plugin_glatitudehistory, 'adminMenuOptions' ) );
	}
	
	register_activation_hook( __FILE__,  array(&$plugin_glatitudehistory, 'install' ));
	register_activation_hook(__FILE__, array(&$plugin_glatitudehistory, 'scheduleCron' ));
	register_deactivation_hook(__FILE__, array(&$plugin_glatitudehistory, 'unscheduleCron' ));
	register_deactivation_hook(__FILE__, array(&$plugin_glatitudehistory, 'dropDatebase' ));
	
	wp_register_script('google_maps', 'http://maps.google.com/maps/api/js?sensor=false', false, '3', true);
	wp_register_script('styled-marker', plugins_url('js/StyledMarker.js', __FILE__), array('google_maps'), '0.5', true);
	wp_register_script('google-latitude-history', plugins_url('js/glatitudehistory.js', __FILE__), array('google_maps', 'styled-marker'), '0.6', true);
	
	add_action('admin_menu', 'glatitudehistory_admin_menu');	
	add_action('hourly_latitude_sync', array(&$plugin_glatitudehistory, 'syncLocations'));
	add_action('admin_notices', array(&$plugin_glatitudehistory, 'oauthCheck'));	
	add_action('wp_footer', array(&$plugin_glatitudehistory, 'printScript'));
	add_shortcode( 'glatitude-map', array(&$plugin_glatitudehistory, 'shortcodeHandler'));
}

class GoogleLatitudeHistory {
	
	const DEFAULT_MAX_LOCATIONS = 20;
	const MAX_LOCATIONS = 100;
	const DEFAULT_CSS = 'margin:0 auto;font-size:12px;line-height:16px;';
	const LATITUDE_TABLE = 'glatitude_history'; 

	// determines if shortcode has been used on a page, if so then print the associated javascript
	static $add_script;
	
	function GoogleLatitudeHistory() {	
		
		//Inserts Latitude OAuth info into the DB
		if(isset($_REQUEST['oauth_token']) && isset($_REQUEST['oauth_verifier'])) {
			
			//Create LatitudeOAuth object with app key/secret and token key/secret from default phase
			$connection = new LatitudeOAuth(get_option('glatitudehistory_oauth_consumer_key'), get_option('glatitudehistory_oauth_consumer_secret'), $_SESSION['oauth_token'], $_SESSION['oauth_token_secret']);
			
			//Request access tokens from google latitude
			$access_token = $connection->getAccessToken($_REQUEST['oauth_verifier']);
			update_option( 'glatitudehistory_oauth_token', $access_token['oauth_token']);
			update_option( 'glatitudehistory_oauth_secret', $access_token['oauth_token_secret']);
			
			header('Location: ' . $_SERVER['SCRIPT_NAME'] . '?page=google-latitude-history-menu');
		}
		
		//Creates Latitude connection 
		$this->latitude_con = new LatitudeOAuth(get_option('glatitudehistory_oauth_consumer_key'), get_option('glatitudehistory_oauth_consumer_secret'), get_option('glatitudehistory_oauth_token'), get_option('glatitudehistory_oauth_secret'));
	}
	
	/**
	* Install Google Latitude History Plugin.
	* Register settings, set defaults, and create database table
	*/
	function install() {
		
		register_setting( 'glatitudehistory-settings-group', 'glatitudehistory_oauth_consumer_key' );
		register_setting( 'glatitudehistory-settings-group', 'glatitudehistory_oauth_consumer_secret' );
		register_setting( 'glatitudehistory-settings-group', 'glatitudehistory_latitude_api_key' );
		register_setting( 'glatitudehistory-settings-group', 'glatitudehistory_granularity' );
		register_setting( 'glatitudehistory-settings-group', 'glatitudehistory_max_locations' );
		register_setting( 'glatitudehistory-settings-group', 'glatitudehistory_location' );
		register_setting( 'glatitudehistory-settings-group', 'glatitudehistory_oauth_token' );
		register_setting( 'glatitudehistory-settings-group', 'glatitudehistory_oauth_token_secret' );
		register_setting( 'glatitudehistory-settings-group', 'glatitudehistory_custom_css' );
		
		// default latitude query parameter for location history
		update_option('glatitudehistory_location', 'all'); 
		// default maximum number of locations
		update_option('glatitudehistory_max_locations', self::DEFAULT_MAX_LOCATIONS);
		// default css for map div
		update_option('glatitudehistory_custom_css', self::DEFAULT_CSS);
				
		$this->createTable();
	}
	
	/**
	* Creates the database tables used to store Google Latitude locations
	*/
	function createTable() {
		global $wpdb;
		
		$sql = "DROP TABLE IF EXISTS ". $wpdb->prefix.self::LATITUDE_TABLE;
		$wpdb->query($sql);
		
   		if($wpdb->get_var("show tables like '".$wpdb->prefix.self::LATITUDE_TABLE."'") != $wpdb->prefix.self::LATITUDE_TABLE) {

   			$sql = "CREATE TABLE " . $wpdb->prefix.self::LATITUDE_TABLE . " (
   				id INT AUTO_INCREMENT PRIMARY KEY,
   				timestamp_ms BIGINT(20) NOT NULL,
   				date VARCHAR(200) NOT NULL,
   				latitude decimal(12,9) NOT NULL,
   				longitude decimal(12,9) NOT NULL,
   				accuracy INT(11) NOT NULL,
   				location_name VARCHAR(512) default NULL,
   				INDEX idx_timestamp_ms (timestamp_ms));";
   			
   			require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
			dbDelta($sql);
   		}
	}
	
	/**
	* Drops the table used to store Google Latitude locations
	*/
	function dropTable() {
		global $wpdb;
		
		$sql = "DROP TABLE IF EXISTS ". $wpdb->prefix.self::LATITUDE_TABLE;
		
		$wpdb->query($sql);
	}
	
	/**
	* Creates sync location cron job
	*/
	function scheduleCron() {
		wp_schedule_event(time(), 'hourly', 'hourly_latitude_sync' );
	}
	
	/**
	* Removes sync location cron job
	*/
	function unscheduleCron() {
		wp_clear_scheduled_hook( 'hourly_latitude_sync' );
	}
		
	/**
	* Checks if the OAuth token is stale by using it to call
	* the Google Latitude rest service for the current location
	*
	* @return string error message for stale token
	*/
	function oauthCheck() {
		
		// get current location
		$currentLocation = $this->latitude_con->get('currentLocation');
		
		// we have a stale token
		if(isset($currentLocation->error)) {
			echo '<div class="error"><p><span style="font-weight:bold">Google Latitude History: </span>Your OAuth token is stale, please refresh the token on the <a href="options-general.php?page=google-latitude-history-menu">settings page</a>. Reason: '.$currentLocation->error->message.'</p></div>';
		}
	}
	
	/**
	*  add google latitude history link to the admin menu
	*/
	function adminMenu() {
		add_options_page('Google Latitude History Settings', 'Google Latitude History', 'manage_options', 'google-latitude-history-menu', array('GoogleLatitudeHistory', 'adminMenuOptions' ) );
	}
	
	/**
	* This function handles the plugin admin settings page which includes
	* updating the settings, syncing the database with Google Latitude, and
	* displaying the settings page 
	*
	* @return string the settings page as html
	*/
	function adminMenuOptions() {
		global $wpdb;
		
		echo( '<h2>Google Latitude History Settings</h2>' );
		
			wp_nonce_field( 'update-options' );

			if(isset($_POST['submit']) && $_POST['submit'] == 'Update Options' ) {

				// Update Google Latitude Settings
				update_option('glatitudehistory_oauth_consumer_key', $_POST['oauth_consumer_key']);
				update_option('glatitudehistory_oauth_consumer_secret', $_POST['oauth_consumer_secret']);
				update_option('glatitudehistory_granularity', $_POST['granularity']);
				update_option('glatitudehistory_custom_css', $_POST['custom_css']);
				
				// check if max locations parameter is a number and gt 0 and lt MAX_LOCATIONS
				// if not then set it to the default number of locations
				update_option('glatitudehistory_max_locations', self::DEFAULT_MAX_LOCATIONS);
				if( is_numeric($_POST['max_locations']) ) {
					$max_locations = (int) $_POST['max_locations'];
					if($max_locations > 0 && $max_locations < self::MAX_LOCATIONS) {
						update_option( 'glatitudehistory_max_locations', $max_locations);
					}
				}
					
				echo( '<div class="updated"><p>Settings Updated</p></div>' );
			
			} elseif(isset($_POST['submit']) && $_POST['submit'] == 'Sync Location History' ) {
				
				$this->syncLocations();
				
				echo( '<div class="updated"><p>Google Latitude History was synced</p></div>' );
			}
			
			// display the settings page
			glatitudehistory_settings();
	}
	
	/**
	* Calls the Google Latitude rest service to get the location history and 
	* updates the database.
	*/
	function syncLocations() {
		// get locations from glatitude, pass max locations parameter
		$latitudeHistory = $this->latitude_con->get('location', array('max-results' => get_option('glatitudehistory_max_locations')));
		
		if(!isset($latitudeHistory->error) && isset($latitudeHistory->data) && isset($latitudeHistory->data->items) ) {
			$this->updateDatabase($latitudeHistory);
		} else {
			echo "Error setting Latitude location history<br/>";
		}
	}
	
	/**
	* Takes the location history and compares it to whats in the 
	* the database then adds new locations and removes old ones
	* from the database
	*
	* @param object location history
	*/
	function updateDatabase($latitudeHistory) {
		global $wpdb;
	
		// create hash of location info by timestamp from glatitude
		// and put the glatitude locations (new locations) in an array for comparison to the database locaations
		$googleTimestamps = array();
		$googleLocationsByTimestamp = array();
		foreach($latitudeHistory->data->items as $location) {
			$googleTimestamps[] = $location->timestampMs;
			$googleLocationsByTimestamp[$location->timestampMs] = array(
				'timestampMs' => $location->timestampMs,
				'latitude' => $location->latitude,
				'longitude' => $location->longitude,
				'accuracy' => $location->accuracy
			);
		}
			
		// put the database locataions ("old locations") into an array for comparison to the glatitude locations
		$wpHistory = $wpdb->get_results('SELECT * FROM ' . $wpdb->prefix.self::LATITUDE_TABLE);
		$wpTimestamps = array();
		foreach($wpHistory as $location) {
			$wpTimestamps[] = $location->timestamp_ms;
		}
		
		// find new locations from glatitude
		$newTimestamps = array_diff($googleTimestamps, $wpTimestamps);
	
		// find old locations in the database
		$oldTimestamps = array_diff($wpTimestamps, $googleTimestamps);
			
		// insert new locations into database
		foreach($newTimestamps as $timestamp) {
			$location = $googleLocationsByTimestamp[$timestamp];
			
			// get location name by reverse code
			$locationName = $this->getLocationName($location['latitude'], $location['longitude']);
			$wpdb->insert( $wpdb->prefix.self::LATITUDE_TABLE, array(
				'timestamp_ms' => $location['timestampMs'], 
				'date' => date("F j, Y, g:i a e", $location['timestampMs']/1000), // format date to "July 7, 2011, 12:58 pm UTC"
				'latitude' => $location['latitude'],
				'longitude' => $location['longitude'],
				'accuracy' => $location['accuracy'],
				'location_name' => $locationName
			), array( '%s', '%s', '%s', '%s', '%d', '%s' ) );
		}
			
		// remove old locations from database
		if(count($oldTimestamps) > 0) {
			$wpdb->query('DELETE FROM '.$wpdb->prefix.self::LATITUDE_TABLE.' WHERE timestamp_ms in ('.join(',', $oldTimestamps). ');');
		}
	}
	
	/**
	* Takes location coordinates and uses Google reverse geocoding service
	* to get the location name
	*
	* @param string latitude
	* @param string longitude
	* @return string location name
	*/
	function getLocationName($lat, $lng) {
		
		// reverse geocode the coordinates to get the location name
		$response = json_decode(file_get_contents('http://maps.googleapis.com/maps/api/geocode/json?sensor=false&latlng='.$lat.','.$lng));
		
		// go through the results to find the formatted address (there are several options)
		$address = array();
		if($response->status == 'OK') {
			foreach($response->results as $result) {
				foreach($result->types as $type) {
					if($type == 'locality') {
						$address['locality'] = $result->formatted_address;
						break;
					} elseif($type == 'administrative_area_level_2') {
						$address['administrative_area_level_2'] = $result->formatted_address;
						break;
					} elseif($type == 'administrative_area_level_1') {
						$address['administrative_area_level_1'] = $result->formatted_address;
						break;
					}
				}
				// most preferred formatted address so let's stop
				if(isset($address['locality'])) {
					break;
				}
			}
		}
		
		// Prefer for formatted address = locality > administrative_area_level_2 > administrative_area_level_1
		$locationName = "";
		if(isset($address['locality'])) {
			$locationName = $address['locality'];
		} elseif(isset($address['administrative_area_level_2'])) {
			$locationName = $address['administrative_area_level_2'];
		} elseif(isset($address['administrative_area_level_1'])) {
			$locationName = $address['administrative_area_level_1'];
		}
	
		return $locationName;
	}
	
	/**
	* The function turns the shortcode into html and javascript for displaying the
	* Google Latitude location history on a Google Map
	*
	* @param array shortcode attributes
	* @return string html and javascript for map
	*/
	function shortcodeHandler($attrs) {
		global $wpdb;
		
		self::$add_script = true;
		
		// last location is the first result
		$latitudeHistory = $wpdb->get_results('SELECT * FROM '.$wpdb->prefix.self::LATITUDE_TABLE.' ORDER BY timestamp_ms DESC');
	
		return GoogleLatitudeHistoryMap::generateMap($attrs, $latitudeHistory, get_option('glatitudehistory_custom_css'));
	}
	
	/**
	* Decides whether the javascript files with be included in a page
	*/
	function printScript() {
		if ( ! self::$add_script )
		return;
	
		// used print instead of enqueue b/c it doesnt work when printing the js on the bases of a variable 
		wp_print_scripts('google_maps');
		wp_print_scripts('styled-marker');
		wp_print_scripts('google-latitude-history');
	}
}	

?>